/***************************************************************************
 *   Copyright (C) 2006 by André Lindhjem <belgarat@sdf.lonestar.org>,     *
 *                         Kjetil Holien <kjetil.holien@gmail.com>,        *
 *                         Terje Risa <terje.risa@gmail.com> &             *
 *                         Øyvind Nerbråten <oyvind@nerbraten.com>         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************
 *
 * \file main.c
 * \author André Lindhjem, Kjetil Holien, Terje Risa & Øyvind Nerbråten
 * \date 21.03.2006
 * 
 * A simple console front-end to demonstrate the use of libdaisy.
 */


#include <unistd.h>
#include <stdio.h>
#include <signal.h>
#include <stdlib.h>
#include <ncursesw/ncurses.h> /* add -lncursesw to build! */
#include <libdaisy.h>
#include <locale.h>   /* Definition of Locale Variables LC_*  */
#include <libintl.h>  /* Function definitions for NLS */

/* static function declarations */
static void printmenu (void);
/* callback function declarations */
void cb_audio_done	(void *);
void cb_audio_next	(void *, unsigned long int);
void cb_text		(void *, void *);
void cb_id			(void *, void *);
void cb_error		(void *, enum daisy_status, const char *);
void cb_progress	(void *, long int);

/* Listens for CTRL-C and does it's best to free resources in a Good Way */
static void sigproc (int);

/* Macro for reducing the typing */
#define _(String) gettext(String)


int main (int argc, char *argv[])
{	
	int keypress = 0;	/* user input */
	daisyplayer_t daisy; /* creating a daisyplayer */
	
	argc = argc; argv = argv; /* we don't use the arguments */
	
	/* assigning the locale to the user's defined locale */
	setlocale(LC_ALL,"");
	/* binds the name "daisyconsole" to the directory root of the message files */
	bindtextdomain ("daisyconsole","/usr/share/locale"); /* default locale */
	bindtextdomain ("daisyconsole", "translations"); /* the local locale directory from the tar ball */
	/* sets the application name */
	textdomain ("daisyconsole");
	
	/* adding signal handling. this should catch ctrl-c and free the resources because this is a Good Thing */
	(void) signal(SIGINT, sigproc);
	
	/* initializing daisyplayer with the callback functions */
	daisy = daisy_init(NULL,
					   cb_audio_done,
					   cb_audio_next,
					   cb_text,
					   cb_id,
					   cb_error,
					   cb_progress);
					   
			   
	initscr ();					/* start the curses mode */
	noecho ();					/* don't echo to screen */
	keypad (stdscr, TRUE);		/* enable keypad keys */
	scrollok(stdscr, TRUE);		/* make the window scroll automatically */
	idlok(stdscr, TRUE);
	
	printw ("\n\n\n\t%s:\n\n", _("Welcome to"));
	printw ("\t\t%s\n\n", _("DaisyPlayer console edition"));
	/*
	printw ("________         .__               __________.__                             ");
	printw ("\\______ \\ _____  |__| _________.__.\\______   \\  | _____  ___.__. ___________ ");
	printw (" |    |  \\\\__  \\ |  |/  ___<   |  | |     ___/  | \\__  \\<   |  |/ __ \\_  __ \\");
	printw (" |    `   \\/ __ \\|  |\\___ \\ \\___  | |    |   |  |__/ __ \\\\___  \\  ___/|  | \\/");
	printw ("/_______  (____  /__/____  >/ ____| |____|   |____(____  / ____|\\___  >__|   ");
	printw ("        \\/     \\/        \\/ \\/                         \\/\\/         \\/       ");
	*/
	printmenu (); /* write the menu to screen */
		
	while (keypress != 'q')		/* q quits */
	{
		char string[1000];
		char *infoline = NULL;
		int ret = 0, i = 0;
		daisy_position *bookmark = NULL;
		keypress = wgetch (stdscr); /* get user input */

		switch (keypress)
		{
			case KEY_UP:
				ret = daisy_seek (daisy, DAISY_SEEK_PREV_CHAPTER); /* seek */
				if (ret != 1)
				{
					printw ("%s\n", _("Seek failed"));
					refresh ();	
				}
				break;
			
			case KEY_DOWN:
				ret = daisy_seek (daisy, DAISY_SEEK_NEXT_CHAPTER); /* seek */
				if (ret != 1)
				{
					if (ret == 0)
						printw ("%s\n", _("End Of Book"));
					else
						printw ("%s\n", _("Seek failed"));
					refresh ();	
				}
				break;
			
			case KEY_LEFT:
				ret = daisy_seek (daisy, DAISY_SEEK_PREV_PASSAGE); /* seek */
				if (ret != 1)
				{
					printw ("%s\n", _("Seek failed"));
					refresh ();	
				}
				break;
			
			case KEY_RIGHT:
				ret = daisy_seek (daisy, DAISY_SEEK_NEXT_PASSAGE); /* seek */
				if (ret != 1)
				{
					if (ret == 0)
						printw ("%s\n", _("End Of Book"));
					else
						printw ("%s\n", _("Seek failed"));
					refresh ();	
				}
				break;
			
			case ' ':
				ret = daisy_pause (daisy); /* pause/restart playback */
				if (ret == 1)
				{
					printw ("%s\n", _("Toggling pause"));	
				} else {
					printw ("&S\n", _("Failed to toggle pause"));
					refresh ();
				}
				break;
			
			case 'p':
				printw ("\n");
				ret = daisy_play (daisy); /* start playback */
				if (ret != 1)
				{
					printw ("%s\n", _("Unable to start playback"));
					refresh ();
				}
				break;
			
			case 's':
				if (daisy_stop (daisy) == 1)
				{
					printw ("%s\n", _("Stoped playback"));
					refresh ();
				} else {
					printw ("%s\n", _("Unable to stop playback"));
					refresh ();
				}
				break;
			
			case 'o':
				echo ();
				printw ("\n%s\n", _("Please enter full path to the Daisy book (*/ncc.html or */*.ncx)"));
				(void) scanw("%s", string); /* get path from user */
				noecho ();
				if (string != NULL)
					ret = daisy_load (daisy, string); /* load supplied daisy dtb */
				else
					ret = -1;
				if (ret == 1)
				{
					printw ("\n%s\n\n\n", _("Book loaded, push 'p' to start playing"));
					refresh ();
				} else {
					printw ("&s\n\n", _("Unable to load book"));
					refresh ();
				}
				
				/* output some book information */
				infoline = daisy_get_info(daisy, DAISY_BOOKINFO_TITLETEXT); /* get the title */
				if (infoline != NULL) printw ("%s\n", infoline); /* print the title */
				infoline = daisy_get_info(daisy, DAISY_BOOKINFO_TOTALTIME); /* get totaltime */
				if (infoline != NULL) printw ("%s: %s\n", _("Total time"), infoline); /* print totaltime */
				refresh (); /* refresh screen */
				break;
			
			case 'b': /* seeks to the beginning of the book */
				printw ("\n\n");
				refresh ();
				ret = daisy_seek (daisy, DAISY_SEEK_TO_BEGINNING);
				if (ret != 1)
				{
					printw ("%s\n", _("Seek failed"));
					refresh ();	
				}
				break;
		
			case 'c': /* get the current playback position */
				bookmark = daisy_get_position(daisy); /* get the position struct */
				if (bookmark != NULL)
					printw ("%s: %i %s: %i\n", _("Chapter"), _("Passage"), bookmark->smilpos, bookmark->nodepos);
				free (bookmark);
				break;
			
			case 'j': /* jump to chapter/passage */
				daisy_stop (daisy);
				bookmark = (daisy_position *) malloc (sizeof (daisy_position)); /* allocate a position struct */
				echo ();
				string[0] = '\0';
				printw ("\n%s?: ", _("Chapter"));
				(void) scanw("%s", string);
				bookmark->smilpos = atoi (string); /* read a smilpos from user */
				string[0] = '\0';
				printw ("%s?: ", _("Passage"));
				(void) scanw("%s", string);
				bookmark->nodepos = atoi (string); /* read a nodepos from user */
				noecho ();
				printw ("\n\n");
				refresh ();
				daisy_goto_position (daisy, bookmark); /* jump to new position */
				free (bookmark);
				break;
				
			case 'l': /* list chapters */
				ret = daisy_get_chapter_count (daisy);
				if (ret == -1)
					break;
				printw ("\n");
				refresh ();
				i = 0;
				while (i<ret) /* print 10 chapter in each loop */
				{
					int key = 0;
					int j = i+10;
					for (; i<=j && i<ret; i++) /* print 10 chapters or whats left if <10 left */
					{
						char *title = daisy_get_chapter_info (daisy, i, DAISY_CHAPTER_TITLE);
						if (title != NULL)
						{
							printw ("%i %s\n", i, title); /* print the chapter name and jump position */
							refresh (); 
						}	
					}
					if (i<ret)	/* if there is more left, print a more line and check user input */
					{
						printw ("---%s---\n", _("more"));
						refresh ();
						key = wgetch (stdscr); /* get user input */
						if (key == 'q')	/* if the user input q, break the loop, else loop */
						{
							break;
						}
					}
				}
				printw ("\n");
				refresh ();
				break;
				
			case 'm': /* reprint the menu */
				printmenu ();
				break;
			
			default:
				printw ("\n%s: %d\n", _("unknown key"), keypress);
				refresh ();
				break;
		}
	}
	
	endwin ();
	
	/* terminates and cleans up after daisyplayer */
	daisy_term (daisy);

	return (1);
}


/**
 * A function for printing the menu to screen.
 */
static void printmenu ()
{
	printw ("\t%s:\n", _("Commands"));
	printw ("\t\to = %s\n", _("open daisy book"));
	printw ("\t\tp = %s\n", _("play"));
	printw ("\t\ts = %s\n", _("stop"));
	printw ("\t\tb = %s\n", _("seek to beginning of the book"));
	printw ("\t\t%s = %s\n", _("space"), _("pause/play"));
	printw ("\t\t%s = %s\n", _("left arrow"), _("prev passage"));
	printw ("\t\t%s = %s\n", _("right arrow"), _("next passage"));
	printw ("\t\t%s = %s\n", _("up arrow"), _("prev chapter"));
	printw ("\t\t%s = %s\n", _("down arrow"), _("next chapter"));
	printw ("\t\tc = %s\n", _("print the current playback position (bookmark)"));
	printw ("\t\tj = %s\n", _("jump to playback position (bookmark)"));
	printw ("\t\tl = %s\n", _("list all chapters"));
	printw ("\t\tm = %s\n", _("print this menu"));
	printw ("\t\tq = %s\n\n", _("quit"));
}


/**
 * The callback functon called everytime the daisy engine is done playing a audio segment.
 * @param notused - not used.
 */
void cb_audio_done (void *data)
{
	data = data;
}


/**
 * This callback function is called everytime the daisy engine starts to play a new audio segment.
 * @param length - a unsigned long int containing the length of the audio segment in ms.
 */
void cb_audio_next (void *data, unsigned long int length)
{
	data = data;
	length = length;
}


/**
 * This callback function is called everytime the daisy engine starts to play a new audio segment, supplying the corresponding text.
 * @param passage - an char* containing the text passage.
 */
void cb_text (void *data, void *passage)
{
	char *_passage = passage;
	data = data;
	printw ("%s\n", _passage);
	refresh (); /* refreshing the screen */
}


/**
 * This callback function is called everytime the daisy enfine start to play a new audio segment, supplying the corresponding id to the text in the xml file.
 * @param id - an char* containing the id of the text passage in the xml file.
 */
void cb_id (void *data, void *id)
{
	/*char *_id = id;*/
	id = id;
	data = data;
}


/**
 * This callback function is called everytime an error occurs in the daisy engine.
 * @param err - an enum daisy_status containing the error code of error type.
 * @param daisy_error_msg - an char* containing the error message. This can be NULL when an message is not needed to explain the error.
 */
void cb_error (void *data, enum daisy_status err, const char *daisy_error_msg)
{
	data = data;
	daisy_error_msg = daisy_error_msg;
	/* printing all error messages for debug purposes */
	/*printw ("DaisyConsole got error! (%x) - (%s)\n", err, daisy_error_msg);*/ /*TODO: Remove this line before release */
	if (err == DAISY_ERROR_PLAYBACK_NO_DTB_LOADED)
		printw ("%s 'o'\n", _("No book loaded. Please load a book by pressing"));
	refresh ();
}


/**
 * This callback function is called during playback, supplying the progress in ms.
 * @param progress_ms - an long int containing the progress in ms.
 */
void cb_progress (void *data, long int progress_ms)
{
	data = data;
	progress_ms = progress_ms;
}

/**
 * Listens for CTRL-C and does it's best to free resources in a Good Way
 */
static void sigproc(int signum)
{
	signum = signum;
	(void) signal(SIGINT, sigproc);	/* reset the signal */
	refresh ();
	endwin ();
	exit (0);
}
